<?php

/**
 * 指定された月の日数を返す。
 * @param $year 年
 * @param $month 月
 */
function countDaysOfMonth($year, $month) {
	return date("t", mktime(0,0,0,$month,1,$year));
}

/**
 * 指定された月の最初の曜日を返す。<br />
 * @param $year 年
 * @param $month 月
 * @return 0=日曜、1=月曜、2=火曜、3=水曜、4=木曜、5=金曜、6=土曜
 */
function getFirstDayOfMonth($year, $month) {
	return date("w", mktime(0,0,0,$month,1,$year));
}

/**
 * 指定された日付が休日かどうか判定する。<br />
 * @param $year 年
 * @param $month 月
 * @param $day 日
 * @param false:平日, string:休日名
 */
function isHoliday($year, $month, $day) {
	$dayMap = array(2,1,7,6,5,4,3);
	$firstDay = getFirstDayOfMonth($year, $month);	// 月の最初の曜日コード
	$firstMonday = $dayMap[$firstDay];	// 最初の月曜日
	$retval = false;
	
	switch($month){
		case 1:
			if (1 == $day) {
				$retval = '元旦';
				break;
			} elseif (2 == $day && 1 == whatDay($day, $firstDay)) {
				$retval = '振替休日';
				break;
			}
			
			// 成人の日
			if($year < 2000) {
				if (15 == $day) {
					$retval = '成人の日';
				} elseif (16 == $day && 1 == whatDay($day, $firstDay)) {
					$retval = '振替休日';
				}
			} else {
				if ($firstMonday + 7 == $day) {
					// 第二月曜日
					$retval = '成人の日';
				}
			}
			break;
			
		case 2:
			// 建国記念日
			if (11 == $day) {
				$retval = '建国記念日';
			} elseif (12 == $day && 1 == whatDay($day, $firstDay)) {
				$retval = '振替休日';
			}
			break;
			
		case 3:
			// 春分の日
			if ($year <= 1979) {
				$spring = floor(20.8357 + ($year - 1980)*0.242194 - floor(($year - 1980)/4));
			} elseif (1979 < $year && $year < 2100) {
				$spring = floor(20.8431 + ($year - 1980)*0.242194 - floor(($year - 1980)/4));
			} else {
				$spring = floor(20.851  + ($year - 1980)*0.242194 - floor(($year - 1980)/4));
			}
			if ($spring == $day) {
				$retval = '春分の日';
			} elseif ($spring + 1 == $day && 1 == whatDay($day, $firstDay)) {
				$retval = '振替休日';
			}
			break;
			
		case 4:
			// 天皇誕生日 or みどりの日
			if (29 == $day) {
				if($year < 1989) {
					$retval = '天皇誕生日';
				} else {
					$retval = 'みどりの日';
				} 
			} elseif (30 == $day && 1 == whatDay($day, $firstDay)) {
				$retval = '振替休日';
			}
			break;
			
		case 5:
			if (3 == $day) {
				// 憲法記念日
				$retval = '憲法記念日';
			} elseif (4 == $day) {
				// 国民の休日
				$retval = '国民の休日';
			} elseif (5 == $day) {
				// 子どもの日
				$retval = '子供の日';
			} elseif (6 == $day) {
				// 振り替え
				if (6 == $firstMonday) {
					$retval = '振替休日';
				}
			}
			
			break;
			
		case 7:
			// 海の日
			if($year > 2002) {
				if ($firstMonday + 14 == $day) {
					// 第三月曜日
					$retval = '海の日';
				}
			} elseif($year > 1994) {
				if (20 == $day) {
					$retval = '海の日';
				} elseif (21 == $day && 1 == whatDay($day, $firstDay)) {
					$retval = '振替休日';
				}
			}
			break;
		case 9:
			// 敬老の日
			if($year < 2003) {
				if (15 == $day) {
					$retval = '敬老の日';
					break;
				} elseif (16 == $day && 1 == whatDay($day, $firstDay)) {
					$retval = '振替休日';
					break;
				}
			} else {
				if ($firstMonday + 14 == $day) {
					// 第三月曜日
					$retval = '敬老の日';
					break;
				}
			}
			
			// 秋分の日
			if ($year <= 1979) {
				$autumn = floor(23.2588 + ($year - 1980)*0.242194 - floor(($year - 1980)/4));
			} elseif (1979 < $year && $year < 2100) {
				$autumn = floor(23.2488 + ($year - 1980)*0.242194 - floor(($year - 1980)/4));
			} else {
				$autumn = floor(24.2488 + ($year - 1980)*0.242194 - floor(($year - 1980)/4));
			}
			if ($autumn == $day) {
				$retval = '秋分の日';
			} elseif ($autumn + 1 == $day && 1 == whatDay($day, $firstDay)) {
				$retval = '振替休日';
			}
			break;
			
		case 10;
			// 体育の日
			if($year < 2000) {
				if (10 == $day) {
					$retval = '体育の日';
				} elseif (11 == $day && 1 == whatDay($day, $firstDay)) {
					$retval = '振替休日';
				}
			} else {
				if ($firstMonday + 7 == $day) {
					// 第二月曜日
					$retval = '体育の日';
				}
			}
			break;
	
		case 11:
			
			if (3 == $day) {
				// 文化の日
				$retval = '文化の日';
			} elseif (4 == $day && 1 == whatDay($day, $firstDay)) {
				$retval = '振替休日';
			}
			
			elseif (23 == $day) {
				// 勤労感謝の日
				$retval = '勤労感謝の日';
			} elseif (24 == $day && 1 == whatDay($day, $firstDay)) {
				$retval = '振替休日';
			}
			break;
			
		case 12:			
			// 天皇誕生日
			if($year > 1988) {
				if (23 == $day) {
					$retval = '天皇誕生日';
				} elseif (24 == $day && 1 == whatDay($day, $firstDay)) {
					$retval = '振替休日';
				}
			}
			break;	
	}
	
	return $retval;
}

/**
 * 指定された日の曜日コードを返す。<br />
 * @param $day 日
 * @param $firstDay その月の最初の曜日の曜日コード
 * @return 0=日曜、1=月曜、2=火曜、3=水曜、4=木曜、5=金曜、6=土曜
 */
function whatDay($day, $firstDay) {
	return	($firstDay + ($day%7) - 1)%7;
}

?>
